import {Fragment, useContext, useEffect, useState} from 'react'

import { Dialog, Transition } from '@headlessui/react'
import { ArrowLeftIcon, EmojiSadIcon, XIcon } from '@heroicons/react/outline';
import { XCircleIcon } from '@heroicons/react/solid';

import { cartContext } from '../../cart/CartStore';
import { updateProductQuantity } from '../../cart/CartActions';
import { setAdhesion } from '../../apis/student';
import { sendTransaction } from '../../apis/money';
import { updateStock } from '../../apis/back';

const Cart = () => {
  let [cartError, setCartError] = useState("");
  const { cartState, dispatch } = useContext(cartContext);

  function renderPrice(price: number): string {
    // Render price with currency and fix displaying of price
    return `${price.toFixed(2).replace(".", ",")} €`;
  }

  async function doValidate() {
    const studentNumber: string = "12345678";
    const userName: string = "UserName";

    // Validate transaction with Money
    const successTransaction = await sendTransaction(studentNumber, userName, cartState.totalPrice);
    if (!successTransaction) {
      console.error("Error during transaction with Money.");
      setCartError("Error during transaction with payement server.");
      // return;
    }

    // Set adhesion in Student
    const adhesionInCart = cartState.cart.find(p => p.name === "Adhésion");
    if (adhesionInCart !== undefined){
      const adhesionSuccess = await setAdhesion(studentNumber);
      if (!adhesionSuccess) {
        console.error("Error during adhesion with Student.");
        setCartError("Error during adhesion with student database.");
        // return;
      }
    }

    // Update stock in Back
    cartState.cart.forEach(product => {
      updateStock(product.id, -(product?.quantity ?? 1));
    });

    // Reset cart error message
    setCartError("");

    // Reset cart
    dispatch({ type: 'SET_CART', payload: [] });
  }

  let [totalPrice, setTotalPrice] = useState(0);
  useEffect(() => {
    // Update cart totalPrice
    let totalPrice = 0;
    cartState.cart.forEach((product) => {
      if (product.quantity) {
        totalPrice += product.price * product.quantity;
      } 
    });
    setTotalPrice(totalPrice);
  }, [cartState]);

  return (
    // S'abonne au contexte
    <Transition.Root show={cartState.showCart} as={Fragment}>
      <Dialog as="div" className="fixed inset-0 overflow-hidden z-40" onClose={() => { dispatch({ type: 'TOGGLE_CART' }) }}>
        <div className="absolute inset-0 overflow-hidden">
          <Transition.Child
            as={Fragment}
            enter="ease-in-out duration-500"
            enterFrom="opacity-0"
            enterTo="opacity-100"
            leave="ease-in-out duration-500"
            leaveFrom="opacity-100"
            leaveTo="opacity-0"
          >
            <Dialog.Overlay className="absolute inset-0 bg-gray-500 bg-opacity-75 transition-opacity" />
          </Transition.Child>
          <div className="fixed inset-y-0 right-0 pl-14 max-w-full flex">
            <Transition.Child
              as={Fragment}
              enter="transform transition ease-in-out duration-500 sm:duration-700"
              enterFrom="translate-x-full"
              enterTo="translate-x-0"
              leave="transform transition ease-in-out duration-500 sm:duration-700"
              leaveFrom="translate-x-0"
              leaveTo="translate-x-full"
            >
              <div className="relative w-screen max-w-md">
                <Transition.Child
                  as={Fragment}
                  enter="ease-in-out duration-500"
                  enterFrom="opacity-0"
                  enterTo="opacity-100"
                  leave="ease-in-out duration-500"
                  leaveFrom="opacity-100"
                  leaveTo="opacity-0"
                >
                  <div className="absolute top-0 left-0 -ml-8 pt-4 pr-2 flex sm:-ml-10 sm:pr-4">
                    <button
                      type="button"
                      className="rounded-md text-gray-300 hover:text-white focus:outline-none focus:ring-2 focus:ring-white"
                      onClick={() => { dispatch({ type: 'TOGGLE_CART' }) }}
                    >
                      <span className="sr-only">Close panel</span>
                      <XIcon className="h-6 w-6" aria-hidden="true" />
                    </button>
                  </div>
                </Transition.Child>
                <div className="h-full flex flex-col py-6 bg-white shadow-xl overflow-y-scroll">
                  <div className="px-4 sm:px-6">
                    <Dialog.Title className="text-3xl ml-4 sm:ml-0 font-extrabold text-gray-900">Panier</Dialog.Title>
                  </div>
                  <div className="mt-1 relative flex-1 px-4 sm:px-6">
                    <div className="bg-white">
                      <div className="">
                        <div className="flow-root">
                          <form className="mt-8 mx-4 sm:mx-0">
                            <div>
                              <ul className="-my-6 divide-y divide-gray-200">
                                {cartState.cart.length <= 0 &&
                                  <div className="text-center w-full mt-10 pb-10">
                                    <EmojiSadIcon className="mx-auto h-12 w-12 text-gray-400" />
                                    <h3 className="mt-2 text-sm font-medium text-gray-900">Aucun produit dans le panier !</h3>
                                    <p className="mt-1 text-sm text-gray-500">Rajoutez-en ^^</p>
                                    <div className="mt-6">
                                      <button
                                        type="button"
                                        onClick={() => dispatch({ type: 'TOGGLE_CART' })}
                                        className="inline-flex items-center px-4 py-2 border border-transparent shadow-sm text-sm font-medium rounded-md text-white bg-indigo-600 hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-indigo-500"
                                      >
                                        <ArrowLeftIcon className="-ml-1 mr-2 h-5 w-5" aria-hidden="true" />
                                        Retourner aux produits
                                      </button>
                                    </div>
                                  </div>
                                }
                                {cartState.cart.map(cartItem => (
                                  <li key={cartItem.name} className="py-6 flex">
                                    <div className="flex-1 flex flex-col">
                                      <div>
                                        <div className="flex justify-between text-base font-medium text-gray-900">
                                          <h3>
                                            <span>{cartItem.name}</span>
                                          </h3>
                                          <p className="">{cartItem.stockPrice && renderPrice(cartItem.stockPrice)}</p>
                                        </div>
                                        <div className="flex justify-between text-sm mt-1 text-gray-500">
                                          <p className="">{cartItem.category}</p>
                                          <span>{renderPrice(cartItem.price)}/unité</span>
                                        </div>
                                      </div>
                                      <div className="flex-1 flex justify-between items-baseline text-sm pt-4">
                                        {/* <p className="text-gray-500">Qty {cartItem.quantity}</p> */}
                                        <div className="flex flex-row items-baseline">
                                          <label htmlFor={`quantity-${cartItem.name}`} className="sr-only">
                                            Quantity, {cartItem.name}
                                          </label>
                                          <input
                                            id={`quantity-${cartItem.name}`}
                                            name={`quantity-${cartItem.name}`}
                                            type="number"
                                            max={cartItem.stock}
                                            min={1}
                                            value={cartItem.quantity}
                                            onChange={(e) => { updateProductQuantity(cartState.cart, dispatch, cartItem, parseInt(e.target.value)) }}
                                            className="max-w-full rounded-md border border-gray-300 py-1.5 text-base leading-5 font-medium text-gray-700 text-left shadow-sm focus:outline-none focus:ring-1 focus:ring-indigo-500 focus:border-indigo-500 sm:text-sm"
                                          /><span className="text-sm pl-1">/ {cartItem.stock} unités</span></div>

                                        <div className="flex">
                                          <button type="button" className="font-medium text-indigo-600 hover:text-indigo-500" onClick={() => updateProductQuantity(cartState.cart, dispatch, cartItem, 0)}>
                                            Supprimer
                                          </button>
                                        </div>
                                      </div>
                                    </div>
                                  </li>
                                ))}
                              </ul>
                            </div>

                            {/* Order summary */}
                            {cartState.cart.length > 0 &&
                              <div className="mt-10 sm:ml-32 sm:pl-6">
                                <div className="bg-gray-50 rounded-lg px-4 py-6 sm:p-6 lg:p-8">
                                  <h2 className="sr-only">Panier</h2>

                                  <div className="flow-root">
                                    <dl className="-my-4 text-sm divide-y divide-gray-200">
                                      <div className="py-4 flex items-center justify-between">
                                        <dt className="text-gray-600">Total : </dt>
                                        <dd className="font-medium text-gray-900">{renderPrice(totalPrice)}</dd>
                                      </div>
                                    </dl>
                                  </div>
                                </div>
                                <div className="mt-10">
                                  <button
                                      onClick={() => doValidate()}
                                      type="button"
                                      className="w-full bg-indigo-600 border border-transparent rounded-md shadow-sm py-3 px-4 text-base font-medium text-white hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-offset-2 focus:ring-offset-gray-50 focus:ring-indigo-500"
                                  >
                                    Valider
                                  </button>
                                </div>
                              </div>
                            }
                            {cartError &&
                              <div className="rounded-md bg-red-50 p-4 mt-4">
                                <div className="flex">
                                  <div className="flex-shrink-0">
                                    <XCircleIcon className="h-5 w-5 text-red-400" aria-hidden="true" />
                                  </div>
                                  <div className="ml-3">
                                    <h3 className="text-sm font-medium text-red-800">{cartError}</h3>
                                  </div>
                                </div>
                              </div>
                            }
                          </form>
                        </div>
                      </div>
                    </div>
                  </div>
                </div>
              </div>
            </Transition.Child>
          </div>
        </div>
      </Dialog>
    </Transition.Root>
  );
};

export default Cart;